//	anacreonui.js
//
//	Implements core Anacreon UI functionality
//	Copyright (c) 2012 Kronosaur Productions, LLC. All Rights Reserved.

var $Style = {
	dlgSelectBackground: "#303030",
	dlgHighlightText: "#D9D9FF",
	dlgFadedText: "#808080",
	dlgErrorText: "#FF4040",
	
	dlgFontScale1: "8pt SansationRegular, Verdana, sans-serif",
	dlgFontScale1Height: 14,
	
	dlgFontScale2: "10pt SansationRegular, Verdana, sans-serif",
	dlgFontScale2Bold: "10pt SansationBold, Verdana, sans-serif",
	dlgFontScale2Height: 15,
	
	//	Map
	
	mapActionPath: "#FFFFB2",					//	H:60  S:30  B:100
	mapDisabledWorldText: "#666666",			//	H:0   S:0   B:40
	mapEnemyHighlight: "#FF6666",				//	H:0   S:60  B:100
	mapEnemyPath: "#b25959",					//	H:0   S:50  B:70
	mapEnemySelection: "#FF8080",				//	H:0   S:50  B:100
	mapEnemyTerritory: "#8f4747",				//	H:0   S:50  B:56
	mapEnemyTerritoryHighlight: "#ed7676",		//	H:0   S:50  B:93
	mapEnemyTradeLine: "#4d2828",				//	H:0   S:48  B:30
	mapEnemyUnit: "#FF8C8C",					//	H:0   S:45  B:100
	mapEnemyWorld: "#8f4747",					//	H:0   S:50  B:56
	mapFriendlyFleet: "#FFFFB2",				//	H:60  S:30  B:100
	mapFriendlyPath: "#ffffe5",					//	H:60  S:10  B:100
	mapFriendlyHighlight: "#bfbfff",			//	H:240 S:25  B:100
	mapFriendlyLAMRange: "#1f1e24",				//	H:250 S:17  B:14	(unused)
	mapFriendlySelection: "#D9D9FF",			//	H:240 S:15  B:100
	mapFriendlyTerritory: "#acacbf",			//	H:240 S:10  B:75
	mapFriendlyTerritoryFaded: "#8a8a99",		//	H:240 S:10  B:60
	mapFriendlyTradeLine: "#505059",			//	H:240 S:10  B:35
	mapFriendlyUnit: "#D9D9FF",					//	H:240 S:15  B:100
	mapFriendlyWorld: "#D9D9FF",				//	H:240 S:15  B:100
	mapFriendlyWorldBoxHighlight: "#7c7ca6",	//	H:240 S:25  B:65
	mapFriendlyWorldBoxNormal: "#7e7e8c",		//	H:240 S:10  B:55
	mapFriendlyWorldBoxFaded: "#676773",		//	H:240 S:10  B:45
	mapFriendlyWorldFaded: "#9595a6",			//	H:240 S:10  B:65
	mapIndependentWorldBox: "#4d4d4d",			//	H:0   S:0   B:30
	mapIndependentWorldText: "#999999",			//	H:0   S:0   B:60
	mapJumpBeaconRange: "rgba(77, 77, 69, 0.25)",//	H:60  S:10  B:50
	mapSafeOrbit: "rgba(77, 77, 69, 0.5)",		//	H:60  S:10  B:50
	mapSelectedTradeLine: "#9595a6",			//	H:240 S:10  B:65
	mapSelectedTradeLineArrow: "#acacbf",		//	H:240 S:10  B:75
	mapSelection: "#FFFFFF",					//	H:0   S:0   B:100
	mapSelectionDefaultRadius: 15,
	mapSelectionLineWidth: 2,
	mapTradeLineArrow: "#7e7e8c",				//	H:60  S:10  B:55
	mapWarningText: "#ffffd9",					//	H:60  S:15  B:100

	//	NavBar

	navBarButton: "#5f5f6e",
	
	//	Info Pane Tiles
	
	cxTilePadding: 4,
	cyTilePadding: 2,
	tileFontVerySmall: "6pt SansationRegular, Verdana, sans-serif",
	tileFontVerySmallHeight: 11,
	tileFontSmall: "8pt SansationRegular, Verdana, sans-serif",
	tileFontSmallHeight: 14,
	tileFontMedium: "10pt SansationRegular, Verdana, sans-serif",
	tileFontMediumBold: "10pt SansationBold, Verdana, sans-serif",
	tileFontMediumHeight: 15,
	tileFontLarge: "12pt SansationRegular, Verdana, sans-serif",
	tileFontLargeBold: "12pt SansationBold, Verdana, sans-serif",
	tileFontLargeHeight: 18,
	tileFontExtraLarge: "14pt SansationRegular, Verdana, sans-serif",
	tileFontExtraLargeBold: "14pt SansationBold, Verdana, sans-serif",
	tileFontExtraLargeHeight: 21,
	tileTextBright: "#FFFFD9",
	tileTextFaded: "#808080",
	tileTextHighlight: "#D9D9FF",
	tileTextNormal: "#A0A0A0",
	tileAccentBackground: "#3C1265",
	tileFadedBackground: "#202020",
	tileNormalBackground: "#404040",
	tileHoverBackground: "#808080",
	tileErrorBackground: "#FF4040",
	};

//	BuildDataSelection ---------------------------------------------------------
//
//	MEMBERS
//
//	class: "buildData"
//	id: The ID of the object that owns us.
//	getInfoPanes(): Returns a list of info panes
//	getSpaceObjectID(): Returns the ID of the space object that we are
//		part of (or null).
//	kind: "spaceObjectPart"
//	name: The name of the object
//
//	industry: Industry (trait) that we're showing
//	industryType: Industry design type.
//	obj: The object that owns us.

function BuildDataSelection (obj, industry)
	{
	this["class"] = "buildData";
	this.id = obj.id;
	this.industry = industry;
	this.industryType = $Anacreon.designTypes[industry.traitID];
	this.kind = "spaceObjectPart";
	this.name = obj.name;
	this.obj = obj;

	this.canBeRenamed = obj.canBeRenamed;
	this.isWorld = obj.isWorld;
	this.sovereignID = obj.sovereignID;
	}

BuildDataSelection.prototype.calcImportStatus = function (obj)
	{
	return this.obj.calcImportStatus(obj);
	}

BuildDataSelection.prototype.canImportFrom = function (worldObj)
	{
	return this.obj.canImportFrom(worldObj);
	}
	
BuildDataSelection.prototype.drawGalacticMapBackground = function (ctx, mapMetrics, uiMode)
	{
	this.obj.drawGalacticMapBackground(ctx, mapMetrics, uiMode);
	}

BuildDataSelection.prototype.drawGalacticMapForeground = function (ctx, mapMetrics, uiMode)
	{
	this.obj.drawGalacticMapForeground(ctx, mapMetrics, uiMode);
	}

BuildDataSelection.prototype.drawGalacticMapSelection = function (ctx)
	{
	MapHelper.paintGalacticMapSelection(ctx, this.obj.mapPosX, this.obj.mapPosY, null);
	}
	
BuildDataSelection.prototype.getInfoPanes = function () 
	{
	var industryPane = {
		tabLabel: $Language.capitalizeTitle(this.industryType.nameDesc),
			
		paneDesc: {
			cxStdTile: 120,
			cyStdTile: 42,
			
			onGetTileList: (function (canvasGrid, data)

			//	data:
			//
			//		obj: The selected object. A BuildDataSelection object.
			
				{
				var tileList = [];
				var obj = data.obj.getSpaceObject();
				var industry = data.obj.industry;
				if (industry == null)
					return tileList;

				InfoPaneHelper.createIndustryInfoTiles(tileList, obj, industry);

				return tileList;
				}),
			},
			
		getCommandList: (function (objSelected)
			{
			var commandList = [];
			var obj = objSelected.getSpaceObject();
			var industryType = objSelected.industryType;
			var isForeign = (obj.sovereignID != $Anacreon.userInfo.sovereignID);

			//	Change allocation

			if (!isForeign
					&& industryType.playerAlloc
					&& industryType.buildComplete == null
					&& $Map.uiMode == null)
				{
				commandList.push({
					label: "Allocation",
					data: { obj: objSelected, industryID: industryType.id },
					onCommand: (function (e)
						{
						var obj = e.data.obj.getSpaceObject();
						var industryID = e.data.industryID;
						worldEditIndustryAllocationDialog(obj, industryID);
						})
					});
				}
			
			//	Destroy
			
			if (!isForeign 
					&& industryType.category == "improvement"
					&& !industryType.npeOnly
					&& industryType.buildTime
					&& $Map.uiMode == null)
				{
				commandList.push({
					label: "Destroy",
					data: { obj: objSelected, industryID: industryType.id },
					onCommand: (function (e)
						{
						var obj = e.data.obj.getSpaceObject();
						var industryID = e.data.industryID;
						worldDestroyImprovement(obj, industryID);
						})
					});
				}

			return commandList;
			}),
		};

	return this.obj.getInfoPanes(industryPane);
	}

BuildDataSelection.prototype.getSpaceObject = function ()
	{
	return this.obj;
	}

BuildDataSelection.prototype.getSpaceObjectID = function () 
	{
	return this.id;
	}

BuildDataSelection.prototype.refreshSelection = function ()
	{
	var obj = $Anacreon.objList[this.id];
	var industry = obj.getTrait(this.industry.traitID);

	if (obj && industry)
		return new BuildDataSelection(obj, industry);
	else
		return null;
	}

//	CanvasGrid Tile Helpers ----------------------------------------------------

var InfoPaneHelper =
	{
	clickBuildData: (function (canvasGrid, tileID, data)
	
	//	data:
	//
	//		obj: The object that owns the industry
	//		industry: Industry info (trait)
	//		resType: Product to change
	
		{
		worldEditProductAllocationDialog(data.obj,
				data.industry,
				data.resType,
				data.curAlloc);
		}),

    clickIndustryData: (function (canvasGrid, tileID, data)
        {
        var newSel = new BuildDataSelection(data.obj, data.industry);
        $Map.selectObject(newSel, 3);
        }),

	clickSellData: (function (cavasGrid, tileID, data)
		{
		worldSellResourcesDialog(data.sellerObj,
				data.buyerObj,
				data.resType);
		}),
		
	clickTechImportData: (function (cavasGrid, tileID, data)
		{
		worldEditTechImportDialog(data.importerObj,
				data.exporterObj,
				data.targetTech);
		}),
		
	clickTradeData: (function (cavasGrid, tileID, data)
		{
		worldEditTradeRouteDialog(data.importerObj,
				data.exporterObj,
				data.resType);
		}),
		
	createImageTile: (function (label, value, image)
		{
		return {
			cyTile: 1,
			data: { label: label, value: value, image: image },
			onPaint: InfoPaneHelper.paintImageTile,
			};
		}),
		
	createIndustryInfoTiles: (function (tileList, obj, industry)
		{
		var i;
        var designationType = $Anacreon.designTypes[obj.designation];
        var industryType = $Anacreon.designTypes[industry.traitID];

        //  Add an initial tile to describe the industry

        tileList.push({
            cxTile: 180,
			cyTile: 160,
			data: { obj: obj, industry: industry },
			onPaint: InfoPaneHelper.paintIndustryTile
            });
					
		//	List all resources we can (and cannot) build

        if (industry.buildData)
            {
			var enableProductAlloc = (industry.buildData.length > 3
					&& (industry.isFixed || industry.isPrimary)
					&& industryType.playerProductAlloc);

			for (i = 0; i < industry.buildData.length; i += 3)
				{
				var resType = $Anacreon.designTypes[industry.buildData[i]];
				var alloc = industry.buildData[i + 1];
				var cannotBuild = industry.buildData[i + 2];
				var isDisabled = false;

				//	If we cannot build something because it is obsolete, skip it entirely.

				if (cannotBuild && cannotBuild[0] == "obsolete")
					continue;
						
				//	Compose text

				var cannotBuildText;
				if (cannotBuild == null)
					{
					var j;
							
					//	Find current production numbers
							
					var productionCount = 0;
					if (industry.productionData)
						{
						for (j = 0; j < industry.productionData.length; j += 3)
							if (industry.productionData[j] == resType.id)
								{
								var production = industry.productionData[j + 1];
								if (production < 10)
									productionCount = $Anacreon.formatNumberAsFloat(production, 1);
								else
									productionCount = $Anacreon.formatNumberAsInteger(production);
								break;
								}
						}
					
					cannotBuildText = "Producing " + productionCount + " " + resType.nameDesc + " per watch.";
					}
				else if (cannotBuild[0] == "res")
					{
					var neededType = $Anacreon.designTypes[cannotBuild[1]];
							
					cannotBuildText = "Import or produce " + neededType.nameDesc + " to build " + resType.nameDesc + ".";
					isDisabled = true;
					}
				else if (cannotBuild[0] == "tech")
					{
					var techLevel = $Anacreon.techLevels[cannotBuild[1]];
							
					cannotBuildText = "World must be " + techLevel.name + " level to produce " + resType.nameDesc + ".";
					isDisabled = true;
					}
				else if (cannotBuild[0] == "improve")
					{
					var neededType = $Anacreon.designTypes[cannotBuild[1]];
							
					cannotBuildText = "Build " + neededType.nameDesc + " to produce " + resType.nameDesc + ".";
					isDisabled = true;
					}
				else if (cannotBuild[0] == "unavailable")
					{
					cannotBuildText = "Unable to produce " + neededType.nameDesc + " on world.";
					isDisabled = true;
					}
				else
					{
					cannotBuildText = "Cannot build: " + cannotBuild[0];
					isDisabled = true;
					}
						
				//	Create tile descriptor
						
				var theTile = {
					cyTile: 160,
					data: 
						{
						obj: obj,
						industry: industry,
						resType: resType,
						value: alloc + "%",
						text: cannotBuildText,
						disabled: isDisabled,
						curAlloc: alloc
						},
					onPaint: InfoPaneHelper.paintBuildDataTile,
					onClick: ((!isDisabled && enableProductAlloc) ? InfoPaneHelper.clickBuildData : null),
					};
						
				tileList.push(theTile);
				}
            }
		}),
		
	createResourceTiles: (function (tileList, resources)
		{
		var i;
		
		if (resources == null)
			return;
			
		//	Add all resources
		
		for (i = 0; i < resources.length; i += 2)
			{
			tileList.push({
				cyTile: 1,
				data: { resType: resources[i], resCount: $Anacreon.formatNumberAsInteger(resources[i + 1]) },
				onPaint: InfoPaneHelper.paintResourceTile,
				});
			}
		}),
		
	createSalesTiles: (function (tileList, buyerObj, sellerObj, purchases)
		{
		var i;

		if (purchases)
			{
			var sellerSovereign = $Anacreon.sovereignList[sellerObj.sovereignID];
			var canSell = sellerSovereign.exportsResources();
			var isAHub = sellerObj.isTradingHub();
			var isReady = (sellerObj.buildComplete == null);

			for (i = 0; i < purchases.length; i += 4)
				{
				var resType = $Anacreon.designTypes[purchases[i]];
				var optimal = purchases[i + 1];
				var price = purchases[i + 2];
				var actual = purchases[i + 3];

				var value = $Anacreon.formatNumberAsInteger(optimal);

				var text;
				if (!canSell)
					text = "Doctrine must be Trade & Enterprise to sell resources.";
				else if (!isAHub)
					text = sellerObj.name + " must be a trading hub to sell resources.";
				else if (!isReady)
					text = sellerObj.name + " is still being built.";
				else if (optimal == 0)
					text = sellerObj.name + " is not selling " + resType.nameDesc + ".";
				else if (price == 0.0)
					text = sellerObj.name + " is selling " + resType.nameDesc + ".";
				else if (actual == null)
					text = sellerObj.name + " sold " +
							$Anacreon.formatNumberAsInteger(optimal) + " per watch (" + $Anacreon.formatNumberAsInteger(price) + " aes' worth).";
				else
					text = sellerObj.name + " sold " +
							$Anacreon.formatNumberAsInteger(actual) + " per watch (" + $Anacreon.formatNumberAsInteger(price) + " aes' worth).";

				var theTile = {
					cyTile: 160,
					data: 
						{
						disabled: (!canSell || !isAHub || !isReady || optimal == 0),
						buyerObj: buyerObj,
						sellerObj: sellerObj,
						resType: resType,
						value: value,
						text: text
						},
					onPaint: InfoPaneHelper.paintBuildDataTile,
					onClick: ((canSell && isAHub && isReady) ? InfoPaneHelper.clickSellData : null),
					};

				tileList.push(theTile);
				}
			}
		}),

	createStatsTile: (function (label, value, stat)
		{
		return {
			cyTile: 1,
			data: { label: label, value: value, stat: stat },
			onPaint: InfoPaneHelper.paintStatsTile,
			};
		}),

	createTechImportTile: (function (tileList, importerObj, exporterObj, importTech)
		{
		var targetTech = Math.floor(importTech[0]);
		var techCount = Math.ceil(importTech[1]);

		var text;
		if (importTech[2] == "lack")
			text = "Not enough capacity on " + exporterObj.name + " to uplift.";
		else if (targetTech < importerObj.techLevel)
			text = importerObj.name + " is too advanced to be influenced by " + exporterObj.name;
		else if (techCount == 0)
			{
			if (importerObj.techLevel < targetTech)
				text = "Establishing relationship with " + exporterObj.name;
			else
				text = importerObj.name + " is already at this level.";
			}
		else
			text = importerObj.name + " is being uplifted " + techCount + (techCount == 1 ? " tech level" : " tech levels");
		
		var theTile = {
				cyTile: 160,
				data:
					{
					disabled: (targetTech < importerObj.techLevel),
					importerObj: importerObj,
					exporterObj: exporterObj,
					targetTech: targetTech,
					text: text,
					},
				onPaint: InfoPaneHelper.paintTechImportTile,
				onClick: InfoPaneHelper.clickTechImportData,
				};

		tileList.push(theTile);
		}),
		
	createTradeResourceTiles: (function (tileList, importerObj, exporterObj, imports)
		{
		var i;

		if (imports)
			{
			for (i = 0; i < imports.length; i += 4)
				{
				var resType = $Anacreon.designTypes[imports[i]];
				var alloc = imports[i + 1];
				var optimal = imports[i + 2];
				var actual = imports[i + 3];

				var value = (Math.round(alloc * 10) / 10) + "%";

				var text;
				if (alloc == 0)
					text = importerObj.name + " does not need " + resType.nameDesc + ".";
				else if (actual == null)
					text = importerObj.name + " is importing " +
							$Anacreon.formatNumberAsInteger(optimal) + " per watch.";
				else
					text = importerObj.name + " wants to import " +
							$Anacreon.formatNumberAsInteger(optimal) + " per watch (only " + $Anacreon.formatNumberAsInteger(actual) + " available).";

				var theTile = {
					cyTile: 160,
					data: 
						{
						disabled: (alloc == 0),
						importerObj: importerObj,
						exporterObj: exporterObj,
						resType: resType,
						value: value,
						text: text
						},
					onPaint: InfoPaneHelper.paintBuildDataTile,
					onClick: InfoPaneHelper.clickTradeData,
					};

				tileList.push(theTile);
				}
			}
		}),

	paintBuildDataTile: (function (ctx, xPos, yPos, cxWidth, cyHeight, data)
	
	//	data:
	//
	//		resType: Type of resource we are building
	//		value: Main display value
	//		text: Explanation text
	//		disabled: Paint faded text
	
		{
		ctx.textBaseline = "top";
		
		var xInner = xPos + $Style.cxTilePadding;
		var yInner = yPos + $Style.cyTilePadding;
		var cxInner = cxWidth - 2 * $Style.cxTilePadding;
		var cyInner = cyHeight - 2 * $Style.cyTilePadding;
		
		//	Paint the name of the resource at the top.
		
		var xText = xInner + (cxInner / 2);
		var yText = yInner;
		ctx.font = $Style.tileFontMedium;
		ctx.fillStyle = (data.disabled ? $Style.tileTextFaded : $Style.tileTextHighlight);
		ctx.textAlign = "center";
		yText += $UI.drawText(ctx, xText, yText, cxInner, $Style.tileFontMediumHeight, data.resType.nameDesc);
		
		//	Paint the resource image
		
		var imageHeight = 32;
		var imageWidth = 64;
		var imageX = xInner + (cxInner - imageWidth) / 2;
		var imageY = yInner + 3 * $Style.tileFontMediumHeight;
		
		ctx.fillStyle = (data.disabled ? "#404040" : "#606060");
		ctx.fillRect(imageX, imageY, imageWidth, imageHeight);
		
		//	Paint the main value
		
		yText = imageY + imageHeight;
		ctx.font = $Style.tileFontExtraLargeBold;
		ctx.fillStyle = (data.disabled ? $Style.tileTextFaded : $Style.tileTextHighlight);
		yText += $UI.drawText(ctx, xText, yText, cxInner, $Style.tileFontExtraLargeHeight, data.value);
		
		//	Paint the text below
		
		if (data.text != null)
			{
			yText += $Style.tileFontSmallHeight;
			
			ctx.font = $Style.tileFontSmall;
			ctx.fillStyle = $Style.tileTextNormal;
			yText += $UI.drawText(ctx, xText, yText, cxInner, $Style.tileFontSmallHeight, data.text);
			}
		
		ctx.textAlign = "left";
		}),

	paintButton: (function (ctx, buttonImage, buttonLabel, xPos, yPos, cxWidth, cyHeight)
		{
		ctx.drawImage(buttonImage, xPos, yPos);

		//	Label

		ctx.fillStyle = "ffffff";
		ctx.font = $Style.tileFontLargeBold;
		ctx.textBaseline = "middle";
		ctx.textAlign = "center";

		var xText = xPos + (cxWidth / 2);
		var yText = yPos + (cyHeight / 2);

		ctx.fillText(buttonLabel, xText, yText);
		}),
		
	paintFeatureTile: (function (ctx, xPos, yPos, cxWidth, cyHeight, data)
	
	//	data:
	//
	//		obj: The object that owns the industry
	//		trait: Trait object
	//		traitType: Trait definitions
	
		{
		var i;
		var obj = data.obj;
		var trait = data.trait;
		var traitType = data.traitType
		
		ctx.textBaseline = "top";
		
		var xInner = xPos + $Style.cxTilePadding;
		var yInner = yPos + $Style.cyTilePadding;
		var cxInner = cxWidth - 2 * $Style.cxTilePadding;
		var cyInner = cyHeight - 2 * $Style.cyTilePadding;
		
		//	Paint the image
		
		var imageHeight = 32;
		var imageWidth = 32;
		var imageX = xInner;
		var imageY = yInner;
		
		ctx.fillStyle = "#606060";
		ctx.fillRect(imageX, imageY, imageWidth, imageHeight);
		
		//	Name of trait
		
		var xText = imageX + imageWidth + $Style.cxTilePadding;
		var yText = yInner;
		var cxText = cxInner - (imageWidth + $Style.cxTilePadding);
		ctx.font = $Style.tileFontMedium;
		ctx.fillStyle = $Style.tileTextHighlight;
		yText += $UI.drawText(ctx, xText, yText, cxText, $Style.tileFontMediumHeight, traitType.nameDesc);
		}),
		
	paintImageTile: (function (ctx, xPos, yPos, cxWidth, cyHeight, data)
		{
		ctx.textBaseline = "top";
		
		var xInner = xPos + $Style.cxTilePadding;
		var yInner = yPos + $Style.cyTilePadding;
		var cxInner = cxWidth - 2 * $Style.cxTilePadding;
		var cyInner = cyHeight - 2 * $Style.cyTilePadding;
		
		var yText = yInner;
		InfoPaneHelper.paintTileLabel(ctx, xInner, yText, data.label);
		yText += $Style.tileFontSmallHeight;
		
		//	Paint the text
		
		ctx.font = $Style.tileFontMedium;
		ctx.fillStyle = $Style.tileTextHighlight;
		$UI.drawText(ctx, xInner, yText, cxInner, $Style.tileFontMediumHeight, data.value);
		
		//	Paint the image
		
		var imageHeight = 32;
		var imageWidth = 32;
		var imageX = xInner + cxInner - imageWidth;
		var imageY = yInner + (cyInner - imageHeight) / 2;
		
		ctx.fillStyle = "#606060";
		ctx.fillRect(imageX, imageY, imageWidth, imageHeight);
		}),
		
	paintIndustryTile: (function (ctx, xPos, yPos, cxWidth, cyHeight, data)
	
	//	data:
	//
	//		obj: The object that owns the industry
	//		industry: Industry info (trait)
	
		{
		function paintLine (x, y, cxWidth, label, value, unit)
			{
			var text;
			if (unit)
				text = $Anacreon.formatNumberAsInteger(value) + " " + (value != 1.0 ? unit + "s" : unit);
			else
				text = $Anacreon.formatNumberAsInteger(value);

			ctx.font = $Style.tileFontSmall;
			ctx.fillStyle = "#A0A0A0";
			ctx.fillText(label, x, y);
				
			ctx.textAlign = "right";
			ctx.fillStyle = "#D9D9FF";
			ctx.fillText(text, x + cxWidth, y);
			ctx.textAlign = "left";
			}

		var i;
		var obj = data.obj;
		var industry = data.industry;
		var industryType = $Anacreon.designTypes[industry.traitID];
		
		ctx.textBaseline = "top";
		
		var xInner = xPos + $Style.cxTilePadding;
		var yInner = yPos + $Style.cyTilePadding;
		var cxInner = cxWidth - 2 * $Style.cxTilePadding;
		var cyInner = cyHeight - 2 * $Style.cyTilePadding;
		
		//	Paint the image
		
		var imageHeight = 32;
		var imageWidth = 32;
		var imageX = xInner;
		var imageY = yInner;
		
		ctx.fillStyle = "#606060";
		ctx.fillRect(imageX, imageY, imageWidth, imageHeight);
		
		//	Industry allocation
		
		if (industry.targetAllocation != null)
			{
			ctx.font = $Style.tileFontLarge;
			ctx.fillStyle = $Style.tileTextHighlight;
			ctx.textAlign = "right";
			ctx.fillText($Anacreon.formatNumberAsInteger(industry.workUnits), xInner + cxInner, yInner);
			
			ctx.font = $Style.tileFontSmall;
			ctx.fillStyle = $Style.tileTextLabel;
			var allocText = (industry.targetAllocation == industry.allocation
					? Math.round(10 * industry.allocation) / 10 + "%"
					: Math.round(10 * industry.allocation) / 10 + "% [" + Math.round(10 * industry.targetAllocation) / 10 + "%]");
			
			ctx.fillText(allocText, xInner + cxInner, yInner + $Style.tileFontLargeHeight);
			ctx.textAlign = "left";
			}
		
		//	Name of industry
		
		var xText = imageX + imageWidth + $Style.cxTilePadding;
		var yText = yInner;
		var cxText = (xPos + cxWidth - (imageWidth + 2 * $Style.cxTilePadding)) - xText;
		ctx.font = $Style.tileFontMedium;
		ctx.fillStyle = $Style.tileTextHighlight;
		yText += $UI.drawText(ctx, xText, yText, cxText, $Style.tileFontMediumHeight, industryType.nameDesc);
		
		//	Building?
		
		if (industry.buildComplete != null)
			{
			var designationType = $Anacreon.designTypes[obj.designation];

			//	Figure out how many milliseconds until we're done
			
			var complete = $Anacreon.percentComplete(industry.buildComplete, industryType.buildTime);
			
			//	Draw
			
			var xCenter = xInner + (cxInner / 2);
			var yCenter = yInner + imageHeight + 2 * $Style.cyTilePadding + 16;
			
			InfoPaneHelper.paintProgress(ctx, xCenter, yCenter, complete);

			//	Draw time remaining

			yText = yCenter + 20;
			xText = xCenter;
			ctx.font = $Style.tileFontExtraLargeBold;
			ctx.fillStyle = $Style.tileTextHighlight;
			ctx.textAlign = "center";
			ctx.fillText($Anacreon.formatDurationDigits(industry.buildComplete - $Anacreon.update),
					xText,
					yText);
			yText += $Style.tileFontExtraLargeHeight;

			//	Draw explanation, if necessary

			var explanation = null;
			if (designationType.role == "sectorCapital"
					&& industryType.role == "administration")
				explanation = "Sector capital is under construction.";

			if (explanation)
				{
				ctx.font = $Style.tileFontMedium;
				ctx.fillStyle = $Style.tileTextNormal;

				yText += $UI.drawText(ctx, xText, yText, cxInner, $Style.tileFontMediumHeight, explanation);
				}

			ctx.textAlign = "left";
			}
		
		//	Industry description
		
		else
			{
			yText = yInner + imageHeight + 2 * $Style.cyTilePadding;

			//	If we have tech export data, then show it now

			if (industry.techExportData)
				{
				paintLine(xInner, yText, cxInner, "capacity", industry.techExportData[0], "tech level");
				yText += $Style.tileFontSmallHeight;

				paintLine(xInner, yText, cxInner, "used", industry.techExportData[1], "tech level");
				yText += $Style.tileFontSmallHeight;

				var available = industry.techExportData[0] - industry.techExportData[1];
				paintLine(xInner, yText, cxInner, "available", available, "tech level");
				yText += $Style.tileFontSmallHeight;
				}
			}
		}),

	paintMessageBox: (function (ctx, xPos, yPos, cxWidth, cyHeight, message, color)
		{
		//	Create a path for the box outline

		var cyCorner = 32;
		var cxCorner = Math.floor(Math.tan(Math.PI / 6.0) * cyCorner);
		ctx.beginPath();
		ctx.moveTo(xPos + cxCorner, yPos);
		ctx.lineTo(xPos + cxWidth, yPos);
		ctx.lineTo(xPos + cxWidth, yPos + cyHeight - cyCorner);
		ctx.lineTo(xPos + cxWidth - cxCorner, yPos + cyHeight);
		ctx.lineTo(xPos, yPos + cyHeight);
		ctx.lineTo(xPos, yPos + cyCorner);
		ctx.closePath();

		//	Fill an opaque area with the color

		ctx.globalAlpha = 0.5;
		ctx.fillStyle = color;
		ctx.fill();

		//	Stroke an outline

		ctx.globalAlpha = 1.0;
		ctx.strokeStyle = color;
		ctx.stroke();

		//	Now paint the text inside

		var cxSpacing = cxCorner + 4;
		var cySpacing = $Style.tileFontMediumHeight / 2;
		var xText = xPos + cxSpacing;
		var yText = yPos + cySpacing;
		var cxText = cxWidth - (2 * cxSpacing);

		ctx.font = $Style.tileFontMedium;
		ctx.fillStyle = $Style.dlgHighlightText;
		ctx.textBaseline = "top";
		ctx.textAlign = "left";
		$UI.drawText(ctx, xText, yText, cxText, $Style.tileFontMediumHeight, message);
		}),

	paintNewsTile: (function (ctx, xPos, yPos, cxWidth, cyHeight, data)
	
	//	data:
	//
	//		obj: The object that owns the industry
	//		entry: News entry
	
		{
		var i;
		var obj = data.obj;
		var entry = data.entry;
		
		ctx.textBaseline = "top";
		
		var xInner = xPos + $Style.cxTilePadding;
		var yInner = yPos + $Style.cyTilePadding;
		var cxInner = cxWidth - 2 * $Style.cxTilePadding;
		var cyInner = cyHeight - 2 * $Style.cyTilePadding;
		
		//	Paint the image
		
		var imageHeight = 24;
		var imageWidth = 24;
		var imageX = xInner;
		var imageY = yInner;
		
		ctx.fillStyle = "#606060";
		ctx.fillRect(imageX, imageY, imageWidth, imageHeight);
		
		//	Text
		
		var xText = imageX + imageWidth + $Style.cxTilePadding;
		var yText = yInner;
		var cxText = cxInner - (imageWidth + $Style.cxTilePadding);
		ctx.font = $Style.tileFontMedium;
		ctx.fillStyle = $Style.tileTextNormal;
		$UI.drawText(ctx, xText, yText, cxText, $Style.tileFontMediumHeight, entry.text);
		}),
		
	paintProgress: (function (ctx, x, y, percentDone, clockRadius, clockStyle)
		{
		var radius = (clockRadius ? clockRadius : 16);
		var style = (clockStyle ? clockStyle : $Style.tileTextHighlight);
		
		//	As radians
		
		var sweep = percentDone * 2 * Math.PI;
		var start = 1.5 * Math.PI;
		var end = start + sweep;

		//	Paint the pie wedge
		
		ctx.beginPath();
		ctx.moveTo(x, y);
		ctx.arc(x, y, radius, start, end, false);
		ctx.closePath();

		ctx.fillStyle = style;
		ctx.fill();

		//	Paint the outline

		ctx.beginPath();
		ctx.arc(x, y, radius, 0, 2 * Math.PI, false);
		ctx.closePath();

		ctx.strokeStyle = style;
		ctx.globalAlpha = 0.5;
		ctx.stroke();
		ctx.globalAlpha = 1;
		}),
		
	paintRebellionInfo: (function (ctx, xPos, yPos, cxWidth, cyHeight, data)
	
	//	data:
	//
	//		obj: The object that owns the industry
	//		rebellion: Rebellion trait
	
		{
		function paintLine (x, y, cxWidth, label, value, unit)
			{
			var text;

			if (typeof(value) == "string")
				text = value;
			else if (unit)
				text = $Anacreon.formatNumberAsInteger(value) + " " + (value != 1.0 ? unit + "s" : unit);
			else
				text = $Anacreon.formatNumberAsInteger(value);

			var xCenter = x + (cxWidth / 2);

			ctx.font = $Style.tileFontMedium;
			ctx.fillStyle = $Style.tileTextNormal;
			ctx.textAlign = "right";
			ctx.fillText(label, xCenter - $Style.cxTilePadding, y);
				
			ctx.fillStyle = $Style.tileTextHighlight;
			ctx.textAlign = "left";
			ctx.fillText(text, xCenter + $Style.cxTilePadding, y);
			}

		var i;
		var obj = data.obj;
		var rebellion = data.rebellion;
		
		ctx.textBaseline = "top";
		
		var xInner = xPos + $Style.cxTilePadding;
		var yInner = yPos + $Style.cyTilePadding;
		var cxInner = cxWidth - 2 * $Style.cxTilePadding;
		var cyInner = cyHeight - 2 * $Style.cyTilePadding;
		
		//	Paint the image
		
		var imageHeight = 24;
		var imageWidth = 24;
		var imageX = xInner;
		var imageY = yInner;
		
		ctx.fillStyle = "#606060";
		ctx.fillRect(imageX, imageY, imageWidth, imageHeight);
		
		//	Text
		
		var xText = imageX + imageWidth + $Style.cxTilePadding;
		var yText = yInner;
		var cxText = cxInner - (imageWidth + $Style.cxTilePadding);
		ctx.font = $Style.tileFontLarge;
		ctx.fillStyle = $Style.tileTextHighlight;
		ctx.fillText("Rebellion", xText, yText);

		yText += $Style.tileFontLargeHeight;

		//	Compute rebellion support

		var support;
		if (rebellion.popularSupport < -3)
			support = "very strong imperial support";
		else if (rebellion.popularSupport > 3)
			support = "unquestioning rebel support";
		else
			support = [
				"strong imperial support",		//	-3
				"imperial support",				//	-2
				"partial imperial support",		//	-1
				"neutral",						//	0
				"rebel support",				//	1
				"strong rebel support",			//	2
				"very strong rebel support",	//	3
				][rebellion.popularSupport + 3];

		//	Paint stats

		paintLine(xText, yText, cxText, "rebel forces", rebellion.rebelForces);
		yText += $Style.tileFontMediumHeight;

		paintLine(xText, yText, cxText, "popular support", support);
		yText += $Style.tileFontMediumHeight;

		paintLine(xText, yText, cxText, "rebellion duration", $Anacreon.formatDuration($Anacreon.update - rebellion.rebellionStart));
		yText += $Style.tileFontMediumHeight;
		}),
		
	paintResourceTile: (function (ctx, xPos, yPos, cxWidth, cyHeight, data)
		{
		ctx.textBaseline = "top";
		
		var xInner = xPos + $Style.cxTilePadding;
		var yInner = yPos + $Style.cyTilePadding;
		var cxInner = cxWidth - 2 * $Style.cxTilePadding;
		var cyInner = cyHeight - 2 * $Style.cyTilePadding;
		
		//	Resource image
		
		var imageWidth = 48;
		var imageHeight = 24;
		var imageX = xInner;
		var imageY = yInner + (cyInner - imageHeight) / 2;
		
		ctx.fillStyle = "#606060";
		ctx.fillRect(imageX, imageY, imageWidth, imageHeight);
		
		//	Name of resource
		
		var yText = yInner;
		ctx.font = $Style.tileFontSmall;
		ctx.fillStyle = $Style.tileTextHighlight;
		yText += $UI.drawText(ctx, 
			xInner, 
			yText, 
			cxInner - (imageWidth + 2 * $Style.cxTilePadding), 
			$Style.tileFontSmallHeight, 
			$Anacreon.designTypes[data.resType].nameDesc);
		
		//	Resource count
		
		var textX = xInner + cxInner;
		var textY = yInner + (cyInner - $Style.tileFontExtraLargeHeight) / 2;
		
		ctx.font = $Style.tileFontExtraLargeBold;
		ctx.fillStyle = $Style.tileTextHighlight;
		ctx.textAlign = "right";
		ctx.fillText(data.resCount, textX, textY);
		ctx.textAlign = "left";
		}),

	paintSmallStat: (function (ctx, xPos, yPos, label, stat, statStyle)
		{
		var labelToPaint = label + ": ";

		if (ctx.textAlign == "left")
			{
			ctx.fillStyle = $Style.tileTextNormal;
			ctx.fillText(labelToPaint, xPos, yPos);

			var cxLabel = ctx.measureText(labelToPaint).width + 4;
			xPos += cxLabel;

			if (statStyle)
				ctx.fillStyle = statStyle;
			else
				ctx.fillStyle = $Style.tileTextHighlight;
			ctx.fillText(stat, xPos, yPos);
			}
		else
			{
			if (statStyle)
				ctx.fillStyle = statStyle;
			else
				ctx.fillStyle = $Style.tileTextHighlight;
			ctx.fillText(stat, xPos, yPos);

			var cxLabel = ctx.measureText(stat).width + 4;
			xPos -= cxLabel;

			ctx.fillStyle = $Style.tileTextNormal;
			ctx.fillText(labelToPaint, xPos, yPos);
			}
		}),

	paintSovereignStatsTile: (function (ctx, xPos, yPos, cxWidth, cyHeight, data)
		{
		function paintLine (label, value)
			{
			ctx.font = $Style.tileFontMedium;
			ctx.fillStyle = "#A0A0A0";
			ctx.fillText(label, xInner, yText);
				
			ctx.textAlign = "right";
			ctx.fillStyle = "#D9D9FF";
			ctx.fillText(value, xInner + cxInner, yText);
			ctx.textAlign = "left";

			yText += $Style.tileFontMediumHeight;
			}

		var sovereign = data.sovereign;
		if (sovereign == null)
			return;

		var stats = sovereign.stats;

		ctx.textBaseline = "top";
		
		var xInner = xPos + $Style.cxTilePadding;
		var yInner = yPos + $Style.cyTilePadding;
		var cxInner = cxWidth - 2 * $Style.cxTilePadding;
		var cyInner = cyHeight - 2 * $Style.cyTilePadding;

		//	Paint the image
		
		var imageHeight = 32;
		var imageWidth = 32;
		var imageX = xInner;
		var imageY = yInner;
		
		ctx.fillStyle = "#606060";
		ctx.fillRect(imageX, imageY, imageWidth, imageHeight);
		
		//	Name of empire
		
		var xText = imageX + imageWidth + $Style.cxTilePadding;
		var yText = yInner;
		var cxText = (xPos + cxWidth - (imageWidth + 2 * $Style.cxTilePadding)) - xText;
		ctx.font = $Style.tileFontMedium;
		ctx.fillStyle = $Style.tileTextHighlight;
		yText += $UI.drawText(ctx, xText, yText, cxText, $Style.tileFontMediumHeight, sovereign.name);

		//	Stats

		yText = yInner + imageHeight + 2 * $Style.cyTilePadding;

		paintLine("founded", $Anacreon.formatAge($Anacreon.update - sovereign.foundedOn));

		if (sovereign.doctrine)
			paintLine("doctrine", $Anacreon.designTypes[sovereign.doctrine].nameDesc);

		paintLine("worlds", stats.worlds);
		paintLine("population", $Anacreon.formatPopulation(stats.population));
		paintLine("tech level", $Anacreon.techLevels[stats.techLevel].name);
		paintLine("imperial might", $Anacreon.formatNumberAsInteger(sovereign.imperialMight));

		if (sovereign.secessionChance != null)
			{
			var chance;
			if (sovereign.secessionChance == 0)
				chance = "none";
			else if (sovereign.secessionChance < 0.01)
				chance = "negligible";
			else if (sovereign.secessionChance < 0.02)
				chance = "very low";
			else if (sovereign.secessionChance < 0.05)
				chance = "low";
			else if (sovereign.secessionChance < 0.1)
				chance = "elevated";
			else if (sovereign.secessionChance < 0.2)
				chance = "high";
			else if (sovereign.secessionChance < 0.5)
				chance = "very high";
			else
				chance = "certain";

			paintLine("risk of secession", chance);
			}
		}),
		
	paintStatsTile: (function (ctx, xPos, yPos, cxWidth, cyHeight, data)
		{
		ctx.textBaseline = "top";
		
		var xInner = xPos + $Style.cxTilePadding;
		var yInner = yPos + $Style.cyTilePadding;
		var cxInner = cxWidth - 2 * $Style.cxTilePadding;
		var cyInner = cyHeight - 2 * $Style.cyTilePadding;
		
		var yText = yInner;
		InfoPaneHelper.paintTileLabel(ctx, xInner, yText, data.label);
		yText += $Style.tileFontSmallHeight;
		
		//	Paint the text
		
		ctx.font = $Style.tileFontMedium;
		ctx.fillStyle = $Style.tileTextHighlight;
		$UI.drawText(ctx, xInner, yText, cxInner, $Style.tileFontMediumHeight, data.value);
		
		//	Paint the stat
		
		var textX = xInner + cxInner;
		var textY = yInner + (cyInner - $Style.tileFontExtraLargeHeight) / 2;
		
		ctx.font = $Style.tileFontExtraLargeBold;
		ctx.fillStyle = $Style.tileTextHighlight;
		ctx.textAlign = "right";
		ctx.fillText(data.stat, textX, textY);
		ctx.textAlign = "left";
		}),
		
	paintStructureTile: (function (ctx, xPos, yPos, cxWidth, cyHeight, data)
	
	//	data:
	//
	//		obj: The object that owns the industry
	//		industry: Industry info (trait)
	
		{
		var i;
		var obj = data.obj;
		var industry = data.industry;
		var industryType = $Anacreon.designTypes[industry.traitID];
		
		ctx.textBaseline = "top";
		
		var xInner = xPos + $Style.cxTilePadding;
		var yInner = yPos + $Style.cyTilePadding;
		var cxInner = cxWidth - 2 * $Style.cxTilePadding;
		var cyInner = cyHeight - 2 * $Style.cyTilePadding;
		
		//	Name of industry
		
		var xText = xInner + (cxInner / 2);
		var yText = yInner;
		var cxText = cxInner;
		ctx.font = $Style.tileFontMedium;
		ctx.fillStyle = $Style.tileTextHighlight;
        ctx.textAlign = "center";
		yText += $UI.drawText(ctx, xText, yText, cxText, $Style.tileFontMediumHeight, industryType.nameDesc);
        ctx.textAlign = "left";
		
		//	Paint the image
		
		var imageHeight = 32;
		var imageWidth = 64;
		var imageX = xInner + (cxInner - imageWidth) / 2;
		var imageY = yInner + 2 * $Style.tileFontMediumHeight;
		
		ctx.fillStyle = "#606060";
		ctx.fillRect(imageX, imageY, imageWidth, imageHeight);
		
		//	Industry allocation
		
		if (industry.targetAllocation != null)
			{
            //  Paint the % allocation

            if (industry.targetAllocation != 0)
                {
		        yText = imageY + imageHeight;
		        ctx.font = $Style.tileFontExtraLargeBold;
		        ctx.fillStyle = (industry.targetAllocation == industry.allocation ? $Style.tileTextHighlight : $Style.tileTextFaded);
                ctx.textAlign = "center";
		        yText += $UI.drawText(ctx, xText, yText, cxInner, $Style.tileFontExtraLargeHeight, industry.targetAllocation + "%");
                }

            //   Paint actual work units


            if (industry.workUnits != 0)
                {
    		    ctx.textAlign = "right";
			    ctx.font = $Style.tileFontSmall;
			    ctx.fillStyle = $Style.tileTextHighlight;

			    ctx.fillText($Anacreon.formatNumberAsInteger(industry.workUnits), 
                        xInner + cxInner,
                        imageY + imageHeight + $Style.tileFontExtraLargeHeight - $Style.tileFontSmallHeight);
                }

            //  Paint actual allocation, if not the same

			if (industry.allocation != industry.targetAllocation)
                {
       		    ctx.textAlign = "left";
			    ctx.font = $Style.tileFontSmall;
			    ctx.fillStyle = $Style.tileTextLabel;

			    ctx.fillText(Math.round(10 * industry.allocation) / 10 + "%", 
                        xInner,
                        imageY + imageHeight + $Style.tileFontExtraLargeHeight - $Style.tileFontSmallHeight);
                }

            ctx.textAlign = "left";
			}
		
		//	Building?
		
		if (industry.buildComplete != null)
			{
			//	Figure out how many milliseconds until we're done
			
			var complete = $Anacreon.percentComplete(industry.buildComplete, industryType.buildTime);
			
			//	Draw
			
			var xCenter = xInner + (cxInner / 2);
			var yCenter = imageY + (imageHeight / 2);
			
			InfoPaneHelper.paintProgress(ctx, xCenter, yCenter, complete);
			}
		}),
		
	paintTacticalCloseUp: (function (ctx, xPos, yPos, cxWidth, cyHeight, data)
	
	//	data: tactical object
	
		{
		ctx.textBaseline = "top";
		
		var xInner = xPos + $Style.cxTilePadding;
		var yInner = yPos + $Style.cyTilePadding;
		var cxInner = cxWidth - 2 * $Style.cxTilePadding;
		var cyInner = cyHeight - 2 * $Style.cyTilePadding;
		
		//	Draw status at the top
		
		var yText = yInner;
		
		ctx.font = $Style.tileFontMedium;
		ctx.fillStyle = $Style.tileTextHighlight;
		$UI.drawText(ctx, xInner, yText, cxInner, $Style.tileFontMediumHeight, data.status);
		}),
		
	paintTechImportTile: (function (ctx, xPos, yPos, cxWidth, cyHeight, data)
	
	//	data:
	//
	//		targetTech: Target tech level
	//		text: Explanation text
	//		disabled: Paint faded text
	
		{
		ctx.textBaseline = "top";
		
		var xInner = xPos + $Style.cxTilePadding;
		var yInner = yPos + $Style.cyTilePadding;
		var cxInner = cxWidth - 2 * $Style.cxTilePadding;
		var cyInner = cyHeight - 2 * $Style.cyTilePadding;
		
		//	Paint the name of the resource at the top.
		
		var xText = xInner + (cxInner / 2);
		var yText = yInner;
		ctx.font = $Style.tileFontMedium;
		ctx.fillStyle = (data.disabled ? $Style.tileTextFaded : $Style.tileTextHighlight);
		ctx.textAlign = "center";
		yText += $UI.drawText(ctx, xText, yText, cxInner, $Style.tileFontMediumHeight, "tech level improvement");
		
		//	Paint the tech level image
		
		var imageHeight = 32;
		var imageWidth = 64;
		var imageX = xInner + (cxInner - imageWidth) / 2;
		var imageY = yInner + 3 * $Style.tileFontMediumHeight;
		
		ctx.fillStyle = (data.disabled ? "#404040" : "#606060");
		ctx.fillRect(imageX, imageY, imageWidth, imageHeight);
		
		//	Paint the main value
		
		yText = imageY + imageHeight;
		ctx.font = $Style.tileFontExtraLargeBold;
		ctx.fillStyle = (data.disabled ? $Style.tileTextFaded : $Style.tileTextHighlight);
		$UI.drawText(ctx, xText, yText, cxInner, $Style.tileFontExtraLargeHeight, $Anacreon.techLevels[data.targetTech].name);
		yText += 2 * $Style.tileFontExtraLargeHeight;
		
		//	Paint the text below
		
		if (data.text != null)
			{
			ctx.font = $Style.tileFontSmall;
			ctx.fillStyle = $Style.tileTextNormal;
			yText += $UI.drawText(ctx, xText, yText, cxInner, $Style.tileFontSmallHeight, data.text);
			}
		
		ctx.textAlign = "left";
		}),

	paintTileLabel: (function (ctx, xPos, yPos, value)
		{
		ctx.fillStyle = "#808080";
		ctx.font = $Style.tileFontSmall;
		ctx.fillText(value, xPos, yPos);
		}),
		
	paintUnitStats: (function (ctx, xPos, yPos, cxWidth, cyHeight, data)
	
	//	data:
	//
	//		unitCount: Number of units
	//		unitType: Design type object
	
		{
		var i;
		
		ctx.textBaseline = "top";
		
		var xInner = xPos + $Style.cxTilePadding;
		var yInner = yPos + $Style.cyTilePadding;
		var cxInner = cxWidth - 2 * $Style.cxTilePadding;
		var cyInner = cyHeight - 2 * $Style.cyTilePadding;
		
		var yText = yInner;
		var cyInc;
		
		//	Draw the unit type
		
		ctx.font = $Style.tileFontMedium;
		ctx.fillStyle = $Style.tileTextHighlight;
		cyInc = $UI.drawText(ctx, 
			xInner, 
			yText, 
			cxInner - (3 * $Style.tileFontExtraLargeHeight), 
			$Style.tileFontMediumHeight, 
			data.unitType.nameDesc);
			
		//	Draw number of units
		
		ctx.font = $Style.tileFontExtraLargeBold;
		ctx.fillStyle = $Style.tileTextHighlight;
		ctx.textAlign = "right";
		ctx.fillText(data.unitCount, xInner + cxInner, yText);
		ctx.textAlign = "left";
		
		yText += Math.max(cyInc, $Style.tileFontExtraLargeHeight) + $Style.tileFontSmallHeight;
		
		//	Draw image
		
		var imageWidth = 128;
		var imageHeight = 64;
		var imageX = xInner + (cxInner - imageWidth) / 2;
		var imageY = yText;
		
		ctx.fillStyle = "#606060";
		ctx.fillRect(imageX, imageY, imageWidth, imageHeight);
		
		yText += imageHeight + $Style.tileFontSmallHeight;
		
		//	Stats
		
		for (i = 0; i < data.unitType.stats.length; i += 2)
			{
			ctx.font = $Style.tileFontSmall;
			ctx.fillStyle = $Style.tileTextNormal;
			ctx.fillText(data.unitType.stats[i], xInner, yText);
			
			ctx.textAlign = "right";
			ctx.fillStyle = $Style.tileTextHighlight;
			ctx.fillText(data.unitType.stats[i + 1], xInner + cxInner, yText);
			ctx.textAlign = "left";
			
			yText += $Style.tileFontSmallHeight;
			}
		}),
		
	paintIconAndText: (function (ctx, x, y, cxWidth, imageDesc, label, text)
		{
		ctx.textBaseline = "top";
		
		//	Paint the icon
		
		var imageHeight = 32;
		var imageWidth = (imageDesc == null ? 1 : 32);
		var imageX = x;
		var imageY = y;

		if (imageDesc == null || imageDesc.length == 0)
			{
			ctx.fillStyle = "#606060";
			ctx.fillRect(imageX, imageY, imageWidth, imageHeight);
			}
		else
			{
			var imageType = $Anacreon.designTypes[imageDesc[0]];
			if (imageType != null)
				{
				ctx.drawImage(imageType.imageElement,
						imageDesc[1],
						imageDesc[2],
						imageDesc[3],
						imageDesc[4],
						imageX,
						imageY,
						imageWidth,
						imageHeight);
				}
			}
		
		//	Paint the label
		
		var xText = x + imageWidth + $Style.cxTilePadding;
		var yText = y;
		InfoPaneHelper.paintTileLabel(ctx, xText, yText, label);
		yText += $Style.tileFontSmallHeight;
		
		//	Paint the text
		
		var cxText = cxWidth - (imageWidth + $Style.cxTilePadding);
		ctx.font = $Style.tileFontMedium;
		ctx.fillStyle = $Style.tileTextHighlight;
		$UI.drawText(ctx, xText, yText, cxText, $Style.tileFontMediumHeight, text);
		}),

	paintIconAndTextAndProgress: (function (ctx, x, y, cxWidth, imageDesc, label, text, progress)
		{
		ctx.textBaseline = "top";
		
		//	Paint the icon
		
		var imageHeight = 32;
		var imageWidth = 32;
		var imageX = x;
		var imageY = y;

		if (imageDesc == null || imageDesc.length == 0)
			{
			ctx.fillStyle = "#606060";
			ctx.fillRect(imageX, imageY, imageWidth, imageHeight);
			}
		else
			{
			var imageType = $Anacreon.designTypes[imageDesc[0]];
			if (imageType != null)
				{
				ctx.drawImage(imageType.imageElement,
						imageDesc[1],
						imageDesc[2],
						imageDesc[3],
						imageDesc[4],
						imageX,
						imageY,
						imageWidth,
						imageHeight);
				}
			}
		
		//	Paint the label
		
		var xText = x + imageWidth + $Style.cxTilePadding;
		var yText = y;
		InfoPaneHelper.paintTileLabel(ctx, xText, yText, label);
		yText += $Style.tileFontSmallHeight;
		
		//	Paint the text
		
		ctx.font = $Style.tileFontMedium;
		ctx.fillStyle = $Style.tileTextNormal;
		var cxText = ctx.measureText(text).width;
		ctx.fillText(text, xText, yText);

		//	Paint progress

		var radius = 12;
		var xCenter = xText + cxText + $Style.cxTilePadding + radius;
		var yCenter = yText;

		InfoPaneHelper.paintProgress(ctx, xCenter, yCenter, progress, radius, $Style.tileTextNormal);
		}),

	paintResourceProductionTile: (function (ctx, xPos, yPos, cxWidth, cyHeight, data)
		{
		function paintLine (label, value, optimal)
			{
			if (optimal != 0)
				{
				var valueText = (value < 10 ? $Anacreon.formatNumberAsFloat(value, 1) : $Anacreon.formatNumberAsInteger(value));

				ctx.font = $Style.tileFontSmall;
				ctx.fillStyle = "#A0A0A0";
				ctx.fillText(label, xInner, yText);

				ctx.font = $Style.tileFontMedium;
				ctx.textAlign = "right";
				ctx.fillStyle = "#D9D9FF";
				if (value == optimal)
					ctx.fillText(valueText, xInner + cxInner, yText);
				else
					{
					var optimalText = (optimal < 10 ? $Anacreon.formatNumberAsFloat(optimal, 1) : $Anacreon.formatNumberAsInteger(optimal));
					ctx.fillText(valueText + " [" + optimalText + "]", xInner + cxInner, yText);
					}

				ctx.textAlign = "left";

				yText += $Style.tileFontMediumHeight;
				}
			}

		var i;
		var obj = data.obj;
		var resData = data.resData;
		
		ctx.textBaseline = "top";
		
		var xInner = xPos + $Style.cxTilePadding;
		var yInner = yPos + $Style.cyTilePadding;
		var cxInner = cxWidth - 2 * $Style.cxTilePadding;
		var cyInner = cyHeight - 2 * $Style.cyTilePadding;
		
		//	Paint the image
		
		var imageHeight = 32;
		var imageWidth = 32;
		var imageX = xInner;
		var imageY = yInner;
		
		ctx.fillStyle = "#606060";
		ctx.fillRect(imageX, imageY, imageWidth, imageHeight);
		
		//	Name of resource
		
		var xText = imageX + imageWidth + $Style.cxTilePadding;
		var yText = yInner;
		var cxText = cxInner - (imageWidth + $Style.cxTilePadding);
		ctx.font = $Style.tileFontMedium;
		ctx.fillStyle = $Style.tileTextHighlight;
		yText += $UI.drawText(ctx, xText, yText, cxText, $Style.tileFontMediumHeight, resData.resType.nameDesc);

		//	Itemized production

		yText = yInner + imageHeight + $Style.cyTilePadding;

		ctx.textBaseline = "alphabetic";
		yText += $Style.tileFontSmallHeight;
		paintLine("produced", resData.produced, resData.producedOptimal);
		paintLine("imported", resData.imported, resData.importedOptimal);
		paintLine("consumed", resData.consumed, resData.consumedOptimal);
		paintLine("exported", resData.exported, resData.exportedOptimal);
		paintLine("available", resData.available, resData.available);
		ctx.textBaseline = "top";
		}),
		
	paintResourceUnit: (function (ctx, x, y, cxResBox, alignment, resType, resCount)
		{
		var cyHeight = 16;
		
		var xPos = x;
		var yPos = y + (cyHeight / 2);
		
		ctx.textBaseline = "middle";
		
		//	Paint image
		
		var imageHeight = 16;
		var imageWidth = 32;
		var imageX = (alignment == "right" ? xPos - imageWidth : xPos);
		var imageY = yPos - (imageHeight / 2);
		
		ctx.fillStyle = "#606060";
		ctx.fillRect(imageX, imageY, imageWidth, imageHeight);
		
		var advance = imageWidth + $Style.cxTilePadding;
		if (alignment == "right")
			advance = -advance;
			
		xPos += advance;
		
		//	Paint the count
		
		ctx.font = $Style.tileFontLarge;
		ctx.fillStyle = $Style.tileTextHighlight;
		ctx.textAlign = alignment;
		ctx.fillText(resCount, xPos, yPos);
		
		advance = ctx.measureText(resCount).width + 2 * $Style.cxTilePadding;
		if (alignment == "right")
			advance = -advance;
			
		xPos += advance;
		
		//	Paint name
		
		ctx.font = $Style.tileFontSmall;
		ctx.fillStyle = $Style.tileTextNormal;
		ctx.fillText(resType.nameDesc, xPos, yPos);
		
		ctx.textAlign = "left";
		}),
		
	paintSmallIconAndText: (function (ctx, x, y, cxResBox, alignment, icon, text)
		{
		var cyHeight = 16;
		
		var xPos = x;
		var yPos = y + (cyHeight / 2);
		
		ctx.textBaseline = "middle";
		
		//	Paint image
		
		var imageHeight = 16;
		var imageWidth = 32;
		var imageX = (alignment == "right" ? xPos - imageWidth : xPos);
		var imageY = yPos - (imageHeight / 2);
		
		ctx.fillStyle = "#606060";
		ctx.fillRect(imageX, imageY, imageWidth, imageHeight);
		
		var advance = imageWidth + $Style.cxTilePadding;
		if (alignment == "right")
			advance = -advance;
			
		xPos += advance;
		
		//	Paint name
		
		ctx.font = $Style.tileFontSmall;
		ctx.fillStyle = $Style.tileTextNormal;
		ctx.textAlign = alignment;
		ctx.fillText(text, xPos, yPos);
		
		ctx.textAlign = "left";
		}),
		
	paintSmallValue: (function (ctx, x, y, cxWidth, label, value)
		{
		ctx.textAlign = "center";
		
		//	Paint the label
		
		var xText = x + (cxWidth / 2);
		var yText = y;
		
		InfoPaneHelper.paintTileLabel(ctx, xText, yText, label);
		yText += $Style.tileFontSmallHeight;

		//	Paint stat
		
		ctx.font = $Style.tileFontMedium;
		ctx.fillStyle = $Style.tileTextHighlight;
		ctx.fillText(value, xText, yText);
		
		ctx.textAlign = "left";
		}),

	paintTechLevelIcon: (function (ctx, techLevel, x, y, cxWidth, cyHeight, backgroundColor)
		{
		var imageInfo = TECH_LEVEL_ICONS[techLevel - 1];
		if (imageInfo)
			{
			if (backgroundColor)
				{
				ctx.beginPath();
				ctx.arc(x + cxWidth / 2, y + cyHeight / 2, cxWidth / 2, 0, 2 * Math.PI, false);
				ctx.closePath();

				ctx.fillStyle = backgroundColor;
				ctx.fill();
				}

			ctx.drawImage($Map.techLevelIcons, 
					imageInfo[0],
					imageInfo[1],
					48,
					48,
					x,
					y,
					cxWidth,
					cyHeight);
			}
		}),
		
	paintTechLevelInfo: (function (ctx, x, y, cxWidth, label, techLevel, text)
		{
		ctx.textBaseline = "top";
		
		//	Paint the image
		
		var imageWidth = 32;
		var imageHeight = 32;
		InfoPaneHelper.paintTechLevelIcon(ctx, techLevel, x, y, imageWidth, imageHeight, "#000000");
		
		//	Paint the label
		
		var xText = x + imageWidth + $Style.cxTilePadding;
		var yText = y;
		InfoPaneHelper.paintTileLabel(ctx, xText, yText, label);
		yText += $Style.tileFontSmallHeight;
		
		//	Paint the text
		
		var cxText = cxWidth - (imageWidth + $Style.cxTilePadding);
		ctx.font = $Style.tileFontMedium;
		ctx.fillStyle = $Style.tileTextHighlight;
		$UI.drawText(ctx, xText, yText, cxText, $Style.tileFontMediumHeight, text);
		}),

	paintValue: (function (ctx, x, y, cxWidth, label, value, extraValue)
		{
		ctx.textAlign = "center";
		
		//	Paint the label
		
		var xText = x + (cxWidth / 2);
		var yText = y;
		
		InfoPaneHelper.paintTileLabel(ctx, xText, yText, label);
		yText += $Style.tileFontSmallHeight;

		//	Paint stat
		
		if (extraValue)
			{
			ctx.textAlign = "left";
			ctx.font = $Style.tileFontExtraLargeBold;
			var cxValue = ctx.measureText(value).width;

			ctx.font = $Style.tileFontMedium;
			var cxExtraValue = ctx.measureText(extraValue).width;

			var cxTotal = cxValue + cxExtraValue;

			var xValue = xText - (cxTotal / 2);

			ctx.fillStyle = $Style.tileTextHighlight;
			ctx.font = $Style.tileFontExtraLargeBold;
			ctx.fillText(value, xValue, yText);

			xValue += cxValue;
			ctx.font = $Style.tileFontMedium;
			ctx.fillText(extraValue, xValue, yText);
			}
		else
			{
			ctx.font = $Style.tileFontExtraLargeBold;
			ctx.fillStyle = $Style.tileTextHighlight;
			ctx.fillText(value, xText, yText);
		
			ctx.textAlign = "left";
			}
		}),

	paintFleetTile: (function (ctx, xPos, yPos, cxWidth, cyHeight, data)

	//	data:
	//		obj: object
	
		{
		var i;
		var obj = data.obj;
		
		ctx.textBaseline = "top";
		
		var xInner = xPos + $Style.cxTilePadding;
		var yInner = yPos + $Style.cyTilePadding;
		var cxInner = cxWidth - 2 * $Style.cxTilePadding;
		var cyInner = cyHeight - 2 * $Style.cyTilePadding;
		
		//	Compute some properties
		
		var forces = obj.getForceComposition();
		var speed = obj.getFTLSpeed();
		if (speed == null)
			speed = "N/A";
		else
			speed = speed + " ly per watch";

		//	Compute status
					
		var status;
		if (obj.battlePlan != null)
			{
			if (obj.battlePlan.objective == "invasion")
				status = "invading " + $Anacreon.objList[obj.anchorObjID].name;
			else
				status = "in combat";
			}
		else if (obj.dest != null)
			status = "in transit";
		else
			status = "at destination";

		//	Compute metrics

		var BBoxCount = 1;
		var cxBBox = 140;

		var CBoxCount = 3;
		var cxCBox = 80;

		var ABoxCount = 1;
		var cxABox = 140;
		
		//	Paint top bar

		var x = xInner;
		var y = yInner;
		
		InfoPaneHelper.paintIconAndText(ctx, x, y, cxBBox, [ ], "sovereign", $Anacreon.sovereignList[obj.sovereignID].name);
		x += cxBBox + $Style.cxTilePadding;
		
		InfoPaneHelper.paintIconAndText(ctx, x, y, cxABox, null, "status", status);
		x += cxABox + $Style.cxTilePadding;

		InfoPaneHelper.paintIconAndText(ctx, x, y, cxBBox, null, "speed", speed);
		x += cxBBox + $Style.cxTilePadding;

		InfoPaneHelper.paintIconAndText(ctx, x, y, cxCBox, null, "space forces", $Anacreon.formatNumberAsFloat(forces.spaceForces / 100.0, 1));
		x += cxCBox + $Style.cxTilePadding;
		
		InfoPaneHelper.paintIconAndText(ctx, x, y, cxCBox, null, "ground forces", $Anacreon.formatNumberAsFloat(forces.groundForces / 100.0, 1));
		x += cxCBox + $Style.cxTilePadding;
		
		//	Paint resources
		
		var resources = obj.resources;
		if (resources)
			{
			//	Start by counting how many resources types we have of each of
			//	three categories.

			var spaceForceCount = 0;
			var groundForceCount = 0;
			var commodityCount = 0;

			for (i = 0; i < resources.length; i += 2)
				{
				var resType = $Anacreon.designTypes[resources[i]];

				if (resType.category == "groundUnit")
					groundForceCount++;
				else if (resType.category == "maneuveringUnit")
					spaceForceCount++;
				else
					commodityCount++;
				}

			//	Start below the header

			var cxResBox = cxInner / 3;
			var cxResBoxSmall = cxInner / 4;
			var cyResBox = 18;
			var yStart = yInner + (cyInner / 4) + $Style.cyTilePadding;
			x = xInner;
			y = yStart;
			var curColumn = 0;

			//	Paint ships
		
			for (i = 0; i < resources.length; i += 2)
				{
				var resType = $Anacreon.designTypes[resources[i]];
				var resCount = $Anacreon.formatNumberAsInteger(resources[i + 1]);
			
				if (resType.category == "maneuveringUnit")
					{
					InfoPaneHelper.paintResourceUnit(ctx, x, y, cxResBox, "left", resType, resCount);
				
					y += cyResBox;
					if (y + cyResBox > yInner + cyInner)
						{
						x += cxResBox;
						y = yStart;
						curColumn++;
						}
					}
				}

			//	If we have ground forces, then paint on the next column

			if (groundForceCount > 0)
				{
				//	Next column

				if ((x == xInner || commodityCount == 0)
						&& curColumn < 2)
					{
					x += cxResBox;
					y = yStart;
					curColumn++;
					}

				for (i = 0; i < resources.length; i += 2)
					{
					var resType = $Anacreon.designTypes[resources[i]];
					var resCount = $Anacreon.formatNumberAsInteger(resources[i + 1]);
			
					if (resType.category == "groundUnit")
						{
						InfoPaneHelper.paintResourceUnit(ctx, x, y, cxResBox, "left", resType, resCount);
				
						y += cyResBox;
						if (y + cyResBox > yInner + cyInner)
							{
							x += cxResBox;
							y = yStart;
							curColumn++;
							}
						}
					}
				}

			//	Commodities are at the last column

			if (commodityCount > 0)
				{
				//	Next column

				if (curColumn != 2)
					{
					x = xInner + 2 * cxResBox;
					y = yStart;
					curColumn = 2;
					}

				for (i = 0; i < resources.length; i += 2)
					{
					var resType = $Anacreon.designTypes[resources[i]];
					var resCount = $Anacreon.formatNumberAsInteger(resources[i + 1]);
			
					if (resType.category != "groundUnit"
							&& resType.category != "maneuveringUnit")
						{
						InfoPaneHelper.paintResourceUnit(ctx, x + cxResBox, y, cxResBox, "right", resType, resCount);
				
						y += cyResBox;
						if (y + cyResBox > yInner + cyInner)
							{
							if (curColumn > 1)
								{
								x -= cxResBoxSmall;
								y = yStart;
								curColumn--;
								}
							}
						}
					}
				}
			}
			
		ctx.textAlign = "left";
		}),
		
	paintWorldTile: (function (ctx, xPos, yPos, cxWidth, cyHeight, data)
	
	//	data:
	//		obj: object
	
		{
		var i;
		var obj = data.obj;
		
		ctx.textBaseline = "top";
		
		var xInner = xPos + $Style.cxTilePadding;
		var yInner = yPos + $Style.cyTilePadding;
		var cxInner = cxWidth - 2 * $Style.cxTilePadding;
		var cyInner = cyHeight - 2 * $Style.cyTilePadding;
		
		var cxCol = (cxWidth - (5 * $Style.cxTilePadding)) / 4;
		var xCol1 = xPos + $Style.cxTilePadding;
		var xCol2 = xCol1 + cxCol + $Style.cxTilePadding;
		var xCol3 = xCol2 + cxCol + $Style.cxTilePadding;
		var xCol4 = xCol3 + cxCol + $Style.cxTilePadding;
		
		var cyCol = cyInner;
		var yCol = yInner;
		
		var cyRow = cyCol / 4;
		
		//	Draw the planet horizon

		var worldClass = $Anacreon.designTypes[obj.worldClass];
		var planetR = cxInner / 2;
		var planetH = cyRow;
		var halfAngle = Math.acos((planetR - planetH) / planetR);
		
		var xPlanet = xPos + (cxWidth / 2);
		var yPlanet = yPos + cyHeight + planetR - planetH;
		
		var centerAngle = 1.5 * Math.PI;
		
		if (worldClass.imageLarge != null)
			{
			var imageType = $Anacreon.designTypes[worldClass.imageLarge[0]];

			ctx.save();
			ctx.beginPath();
			ctx.arc(xPlanet, 
					yPos + cyHeight - planetH + (worldClass.imageLarge[4] / 2), 
					worldClass.imageLarge[4] / 2,
					0,
					2 * Math.PI,
					false);
			ctx.clip();

			ctx.drawImage(imageType.imageElement,
					worldClass.imageLarge[1],
					worldClass.imageLarge[2],
					worldClass.imageLarge[3],
					planetH,
					xPlanet - (worldClass.imageLarge[3] / 2),
					yPos + cyHeight - planetH,
					worldClass.imageLarge[3],
					planetH);

			ctx.restore();
			}
		else
			{
			ctx.beginPath();
			ctx.arc(xPlanet, yPlanet, planetR, centerAngle - halfAngle, centerAngle + halfAngle, false);
			ctx.closePath();

			ctx.fillStyle = "#D9D9FF";
			ctx.fill();
			}

		//	Compute metrics

		var BBoxCount = 2;
		var cxBBox = 140;

		var CBoxCount = 3;
		var cxCBox = 80;

		var DBoxCount = 2;
		var cxDBox = 64;

		var cxRemainder = cxInner -
				((cxBBox * BBoxCount)
					+ (cxCBox * CBoxCount)
					+ (cxDBox * DBoxCount)
					+ ($Style.cxTilePadding * (BBoxCount + CBoxCount + DBoxCount - 1)));

		var ABoxCount = 1;
		var cxABox = (cxRemainder - (ABoxCount * $Style.cxTilePadding)) / ABoxCount;
		
		//	Paint top bar

		var longBoxCount = 3;
		var shortBoxCount = 5;
		var cxShortBox = 64;
		var cxLastSection = (shortBoxCount * cxShortBox) + ((shortBoxCount - 1) * $Style.cxTilePadding);
		
		var cxStatsBox = (cxInner - cxLastSection - ((longBoxCount - 1) * $Style.cxTilePadding)) / longBoxCount;
		var x = xInner;
		var y = yInner;
		
		InfoPaneHelper.paintIconAndText(ctx, x, y, cxBBox, [ ], "sovereign", $Anacreon.sovereignList[obj.sovereignID].name);
		x += cxBBox + $Style.cxTilePadding;
		
		var designationType = $Anacreon.designTypes[obj.designation];
		if (obj.buildComplete)
			InfoPaneHelper.paintIconAndTextAndProgress(ctx, x, y, cxABox, designationType.imageSmall, "designation", designationType.nameDesc, $Anacreon.percentComplete(obj.buildComplete, obj.buildTime));
		else
			InfoPaneHelper.paintIconAndText(ctx, x, y, cxABox, designationType.imageSmall, "designation", designationType.nameDesc);
		x += cxABox + $Style.cxTilePadding;
		
        var techLevelValue = $Anacreon.techLevels[obj.techLevel].name + " [" + obj.techLevel;
        if (obj.targetTechLevel)
            {
            if (obj.targetTechLevel > obj.techLevel)
				techLevelValue = techLevelValue + " \u2191" + obj.targetTechLevel;
			else
				techLevelValue = techLevelValue + " \u2193" + obj.targetTechLevel;
            }
		techLevelValue = techLevelValue + "]";

		var forces = obj.getForceComposition();

		var popValue = SpaceObject.populationValue(obj.population);
		var popTargetValue = (obj.targetPopulation ? SpaceObject.populationValue(obj.targetPopulation) : null);
		if (popTargetValue != null && popTargetValue != popValue)
			{
			if (obj.targetPopulation > obj.population)
				popTargetValue = " [\u2191" + popTargetValue + "]";
			else
				popTargetValue = " [\u2193" + popTargetValue + "]";
			}
		else
			popTargetValue = null;

		InfoPaneHelper.paintTechLevelInfo(ctx, x, y, cxBBox, "tech level", obj.techLevel, techLevelValue);
		x += cxBBox + $Style.cxTilePadding;
		
		InfoPaneHelper.paintIconAndText(ctx, x, y, cxCBox, null, "space forces", $Anacreon.formatNumberAsFloat(forces.spaceForces / 100.0, 1));
		x += cxCBox + $Style.cxTilePadding;
		
		InfoPaneHelper.paintIconAndText(ctx, x, y, cxCBox, null, "ground forces", $Anacreon.formatNumberAsFloat(forces.groundForces / 100.0, 1));
		x += cxCBox + $Style.cxTilePadding;
		
		InfoPaneHelper.paintIconAndText(ctx, x, y, cxCBox, null, "social order", obj.revIndex);
		x += cxCBox + $Style.cxTilePadding;
		
		InfoPaneHelper.paintValue(ctx, x, y, cxDBox, "population", popValue, popTargetValue);
		x += cxDBox + $Style.cxTilePadding;
		
		InfoPaneHelper.paintValue(ctx, x, y, cxDBox, "efficiency", obj.efficiency + "%");
		x += cxDBox + $Style.cxTilePadding;
		
		//	Paint defenses & ground forces
		
		var cxResBox = cxCol;
		var cyResBox = 18;
		var xRight = xPos + (cxWidth / 2) - ($Style.cxTilePadding);
		var xLeft = xPos + (cxWidth / 2) + ($Style.cxTilePadding);
		x = xRight;
		y = yInner + cyInner - (planetH + $Style.cyTilePadding) - cyResBox;
		var resAlignment = "right";
		
		var resources = obj.resources;
		if (resources)
			{
			for (i = 0; i < resources.length; i += 2)
				{
				var resType = $Anacreon.designTypes[resources[i]];
				var resCount = $Anacreon.formatNumberAsInteger(resources[i + 1]);
			
				if (resType.category == "fixedUnit" || resType.category == "orbitalUnit" || resType.category == "groundUnit")
					{
					InfoPaneHelper.paintResourceUnit(ctx, x, y, cxResBox, resAlignment, resType, resCount);
				
					if (resAlignment == "right")
						{
						x = xLeft;
						resAlignment = "left";
						}
					else
						{
						x = xRight;
						resAlignment = "right";
						y -= cyResBox;
						}
					}
				}

			//	Paint ships
		
			x = xInner;
			y = yInner + cyRow + $Style.cyTilePadding;
		
			for (i = 0; i < resources.length; i += 2)
				{
				var resType = $Anacreon.designTypes[resources[i]];
				var resCount = $Anacreon.formatNumberAsInteger(resources[i + 1]);
			
				if (resType.category == "maneuveringUnit" || resType.category == "LAMUnit")
					{
					InfoPaneHelper.paintResourceUnit(ctx, x, y, cxResBox, "left", resType, resCount);
				
					y += cyResBox;
					}
				}
			}
		else
			{
			x = xPos + (cxWidth / 2);
			y = yInner + cyInner - (planetH + $Style.cyTilePadding) - cyResBox;

			ctx.font = $Style.tileFontExtraLargeBold;
			ctx.fillStyle = $Style.tileTextHighlight;
			ctx.textBaseline = "bottom";
			ctx.textAlign = "center";
			ctx.fillText("?", x, y);
			ctx.textAlign = "left";
			}
			
		//	Paint features
		
		x = xInner + cxInner;
		y = yInner + cyInner - cyResBox;
		
		for (i = 0; i < obj.traits.length; i++)
			{
			var trait = obj.traits[i];
			var traitType = $Anacreon.designTypes[trait.traitID];
			
			if (traitType.category == "feature" && !traitType.hidden)
				{
				InfoPaneHelper.paintSmallIconAndText(ctx, x, y, cxResBox, "right", null, traitType.nameDesc);
				
				y -= cyResBox;
				}
			}
			
		//	Paint world class
		
		y += (cyResBox - ($Style.tileFontMediumHeight + $Style.cyTilePadding));
		
		ctx.font = $Style.tileFontMedium;
		ctx.fillStyle = $Style.tileTextHighlight;
		ctx.textBaseline = "top";
		ctx.textAlign = "right";
		ctx.fillText($Anacreon.designTypes[obj.worldClass].nameDesc, x, y);
		ctx.textAlign = "left";
		}),
	};

//	Language Helper ------------------------------------------------------------

var LanguageHelper =
	{
	};

//	Map Helper -----------------------------------------------------------------

var MapHelper =
	{
	hitTestGalacticMapSidePane: (function (xPos, yPos, obj)
		{
		//	Compute some metrics

		var maxRadius = $Map.maxWorldRadius;
		var worldRadius = maxRadius * obj.getSizeAdj();
		var outerRadius = worldRadius * 1.2;
		var inner = outerRadius + 10;

		var paneWidth = (obj.sidePaneMetrics ? obj.sidePaneMetrics.width : 150 + inner);
		var paneHeight = (obj.sidePaneMetrics ? obj.sidePaneMetrics.height : 3 * $Style.tileFontSmallHeight + 2 * $Style.cyTilePadding + 20);

		var xLeft = obj.mapPosX - paneWidth;
		var xRight = obj.mapPosX;
		var yTop = obj.mapPosY - paneHeight;
		var yBottom = obj.mapPosY;

		return (xPos >= xLeft && xPos < xRight && yPos >= yTop && yPos < yBottom);
		}),

	paintGalacticMapSidePane: (function (ctx, obj, text, cmdIcon, cmdLabel, isHovering)
		{
		var xCenter = obj.mapPosX;
		var yCenter = obj.mapPosY;

		//	Compute some metrics

		var maxRadius = $Map.maxWorldRadius;
		var worldRadius = maxRadius * obj.getSizeAdj();
		var outerRadius = worldRadius * 1.2;
		var inner = outerRadius + 10;

		var paneWidth = 150 + inner;
		var cxText = paneWidth - 4 * $Style.cxTilePadding;

		//	Compute the number of lines in the text

		ctx.font = $Style.tileFontSmall;
		var lineCount = Math.max(3, $UI.drawTextMeasure(ctx, cxText, $Style.tileFontSmallHeight, text).lines);

		//	If we have too many lines, then make the pane wider

		if (lineCount > 9)
			{
			paneWidth = 300 + inner;
			cxText = paneWidth - 4 * $Style.cxTilePadding;
			lineCount = $UI.drawTextMeasure(ctx, cxText, $Style.tileFontSmallHeight, text).lines;
			}

		var paneHeight = lineCount * $Style.tileFontSmallHeight + 4 * $Style.cyTilePadding + 20;

		//	Remember the pane height in the object (so we can hit test)

		obj.sidePaneMetrics = { height: paneHeight, width: paneWidth };

		//	Draw background
		
		var halfGap = 4;
		var startArc = 1.5 * Math.PI - (Math.sin(halfGap / inner));
		var endArc = Math.PI;
	
		ctx.save();
		ctx.beginPath();
		ctx.arc(xCenter, yCenter, inner, startArc, endArc, true);
		ctx.lineTo(xCenter - paneWidth, yCenter);
		ctx.lineTo(xCenter - paneWidth, yCenter - paneHeight);
		ctx.lineTo(xCenter - halfGap, yCenter - paneHeight);
		ctx.closePath();

		ctx.fillStyle = "#404040";
		ctx.fill();

		//	Clip to the background shape

		ctx.clip();

		//	Text metrics
		
		var xText = xCenter - paneWidth + 2 * $Style.cxTilePadding;
		var yText = yCenter - paneHeight + 2 * $Style.cyTilePadding;

		//	Zoom in button

		ctx.fillStyle = (isHovering ? $Style.tileHoverBackground : "#606060");
		ctx.fillRect(xCenter - paneWidth, yCenter - 20, paneWidth, 20);

		if (cmdIcon)
			$Map.drawIcon(ctx, cmdIcon, xCenter - paneWidth, yCenter - 20);

		ctx.fillStyle = $Style.tileTextNormal;
		ctx.font = $Style.tileFontSmall;
		ctx.textBaseline = "middle";
		ctx.textAlign = "left";
		ctx.fillText(cmdLabel, xCenter - paneWidth + 24, yCenter - 10);
	
		//	Draw text

		ctx.fillStyle = $Style.tileTextBright;
		ctx.font = $Style.tileFontSmall;
		ctx.textBaseline = "top";
		$UI.drawText(ctx, xText, yText, cxText, $Style.tileFontSmallHeight, text);

		ctx.restore();
		}),

	paintGalacticMapSelection: (function (ctx, x, y, radius)
		{
		if (radius == null)
			radius = $Style.mapSelectionDefaultRadius;

		ctx.strokeStyle = $Style.mapSelection;
		ctx.lineWidth = $Style.mapSelectionLineWidth;
		ctx.lineCap = "square";
		
		var arcSize = 0.33 * Math.PI;
		var arcInc = 0.5 * Math.PI;

		//	Draw four arcs around object
		
		var angle = 0.0;
		ctx.beginPath();
		ctx.arc(x, y, radius, angle, angle + arcSize, false);
		ctx.stroke();
		
		angle += arcInc;
		ctx.beginPath();
		ctx.arc(x, y, radius, angle, angle + arcSize, false);
		ctx.stroke();
		
		angle += arcInc;
		ctx.beginPath();
		ctx.arc(x, y, radius, angle, angle + arcSize, false);
		ctx.stroke();
		
		angle += arcInc;
		ctx.beginPath();
		ctx.arc(x, y, radius, angle, angle + arcSize, false);
		ctx.stroke();
		}),

	paintTacticalBoxSelection: (function (ctx, xLeft, yTop, xRight, yBottom, isEnemy)
		{
		var radius = 5;
		ctx.strokeStyle = (isEnemy ? $Style.mapEnemySelection : $Style.mapFriendlySelection);
		ctx.lineWidth = 2;
		ctx.lineCap = "square";
		
		var arcSize = 0.33 * Math.PI;
		var arcInc = 0.5 * Math.PI;
		var angle = (arcInc - arcSize) / 2;

		//	Draw four arcs around the corners
		
		ctx.beginPath();
		ctx.arc(xRight, yBottom, radius, angle, angle + arcSize, false);
		ctx.stroke();
		
		angle += arcInc;
		ctx.beginPath();
		ctx.arc(xLeft, yBottom, radius, angle, angle + arcSize, false);
		ctx.stroke();
		
		angle += arcInc;
		ctx.beginPath();
		ctx.arc(xLeft, yTop, radius, angle, angle + arcSize, false);
		ctx.stroke();
		
		angle += arcInc;
		ctx.beginPath();
		ctx.arc(xRight, yTop, radius, angle, angle + arcSize, false);
		ctx.stroke();

		//	Draw lines

		var radius2 = 2 * radius;

		ctx.beginPath();
		if (Math.abs(xLeft - xRight) > radius2)
			{
			ctx.moveTo(xLeft + radius, yTop - radius);
			ctx.lineTo(xRight - radius, yTop - radius);

			ctx.moveTo(xLeft + radius, yBottom + radius + 1);
			ctx.lineTo(xRight - radius, yBottom + radius + 1);
			}

		if (Math.abs(yTop - yBottom) > radius2)
			{
			ctx.moveTo(xRight + radius + 1, yTop + radius);
			ctx.lineTo(xRight + radius + 1, yBottom - radius);

			ctx.moveTo(xLeft - radius, yTop + radius);
			ctx.lineTo(xLeft - radius, yBottom - radius);
			}

		ctx.stroke();
		}),

	paintTacticalSelection: (function (ctx, x, y, isEnemy)
		{
		var radius = 5;
		ctx.strokeStyle = (isEnemy ? $Style.mapEnemySelection : $Style.mapFriendlySelection);
		ctx.lineWidth = 2;
		ctx.lineCap = "square";
		
		var arcSize = 0.33 * Math.PI;
		var arcInc = 0.5 * Math.PI;
		var angle = (arcInc - arcSize) / 2;

		//	Draw four arcs around object
		
		ctx.beginPath();
		ctx.arc(x, y, radius, angle, angle + arcSize, false);
		ctx.stroke();
		
		angle += arcInc;
		ctx.beginPath();
		ctx.arc(x, y, radius, angle, angle + arcSize, false);
		ctx.stroke();
		
		angle += arcInc;
		ctx.beginPath();
		ctx.arc(x, y, radius, angle, angle + arcSize, false);
		ctx.stroke();
		
		angle += arcInc;
		ctx.beginPath();
		ctx.arc(x, y, radius, angle, angle + arcSize, false);
		ctx.stroke();
		}),

	paintTimeLeftBox: (function (ctx, x, y, watchesLeft)
		{
		ctx.save();

		//	If less than 150 watches then we express the time in watches

		var label;
		if (watchesLeft < 60)
			{
			label = watchesLeft;
			var cxHalfText = (ctx.measureText(label).width / 2);

			ctx.lineWidth = $Style.dlgFontScale2Height + 2;
			ctx.lineCap = "round";

			ctx.beginPath();
			ctx.moveTo(x - cxHalfText, y);
			ctx.lineTo(x + cxHalfText, y);
			ctx.stroke();
			}

		//	Express in cycles

		else
			{
			var label = $Anacreon.formatDurationDigits(watchesLeft);
			var cxHalfText = (ctx.measureText(label).width / 2);
			var cyMargin = 2;
			var cxMargin = 12;

			var xLeft = x - (cxHalfText + cxMargin);
			var xRight = x + (cxHalfText + cxMargin);
			var yTop = y - (($Style.dlgFontScale2Height / 2) + cyMargin);
			var yBottom = y + (($Style.dlgFontScale2Height / 2) + cyMargin);

			var cyShort = cyMargin * 2;
			var cyLong = (yBottom - yTop) - cyShort;
			var cxLong = cyLong * 0.5773;	//	tan (30 degrees)

			ctx.beginPath();
			ctx.moveTo(xLeft, yBottom);
			ctx.lineTo(xLeft, yBottom - cyShort);
			ctx.lineTo(xLeft + cxLong, yTop);
			ctx.lineTo(xRight, yTop);
			ctx.lineTo(xRight, yTop + cyShort);
			ctx.lineTo(xRight - cxLong, yBottom);
			ctx.closePath();
			ctx.fill();
			}

		ctx.font = $Style.dlgFontScale2Bold;
		ctx.fillStyle = "#000000";
		ctx.textAlign = "center";
		ctx.textBaseline = "middle";
		ctx.fillText(label, x, y);

		ctx.restore();
		}),
	};

